/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _PRINT_H
#define _PRINT_H

#include "../result.h"

static FILE *log_stream;

/* Open the log file */
static int open_log_stream(char *logfile)
{
	log_stream = fopen(logfile, "a");
	return log_stream ? 0 : -1;
}


/* print an error */
static int print_error(char *name)
{
	fprintf(stdout,
		"A correct call to %s failed. The test suite will now terminate.\n",
		name);
	fflush(stdout);
}

/* print a mesasge */
static int print_message(char *msg)
{
	if (VERBOSE == 1) {
		fprintf(stdout, "%s\n", msg);
		fflush(stdout);
	}
	fprintf(log_stream, "%s\n", msg);
}

/* get current mode */
static char *get_mode()
{
	if (mode == 0) {
		return "blocking";
	} else {
		return "non-blocking";
	}
}

/* print test name, return values, and test result */
static int print_macro(int test, char *exp, char *name, int ret_value,
		       int ret_expected, int ret_errno)
{
	char *result;
	if (ret_value == ret_expected && errno == ret_errno) {
		result = "PASS";
		if (VERBOSE == 1) {
			fprintf(stdout,
				"%-125s : %-12s : %s : %-3d : %s\n", name,
				get_mode(), result, errno,
				strerror(errno));
			fflush(stdout);
		}
	} else {
		result = "FAIL";
		if (ret_value == ret_expected) {
			fprintf(stdout,
				"%-125s : %-12s : %s : %-3d : %s\n", name,
				get_mode(), result, errno,
				strerror(errno));
		} else {
			fprintf(stdout,
				"%-125s : %-12s : %s : ret = %d : %-3d : %s\n",
				name, get_mode(), result, ret_value, errno,
				strerror(errno));
		}
		fflush(stdout);
	}
	fprintf(log_stream, "%-125s : %-12s : %s : %d : %-3d : %s\n", name,
		get_mode(), result, ret_value, errno, strerror(errno));
}

/* macro modified for exs_mhandle_t */
static int print_mhandle_macro(int test, char *exp, char *name,
			       exs_mhandle_t ret_value, int ret_errno)
{
	char *result;
	if (ret_value == EXS_MHANDLE_INVALID && errno == ret_errno) {
		result = "PASS";
		if (VERBOSE == 1) {
			fprintf(stdout,
				"%-125s : %-12s : %s : %-3d : %s\n", name,
				get_mode(), result, errno,
				strerror(errno));
			fflush(stdout);
		}
	} else {
		result = "FAIL";
		fprintf(stdout, "%-125s : %-12s : %s : %-3d : %s\n", name,
			get_mode(), result, errno, strerror(errno));
		fflush(stdout);
	}
	fprintf(log_stream, "%-125s : %-12s : %s : %d : %-3d : %s\n", name,
		get_mode(), result, ret_value, errno, strerror(errno));
}

/* macro modified for exs_qhandle_t */
static int print_qhandle_macro(int test, char *exp, char *name,
			       exs_qhandle_t ret_value, int ret_errno)
{
	char *result;
	if (ret_value == EXS_QHANDLE_INVALID && errno == ret_errno) {
		result = "PASS";
		if (VERBOSE == 1) {
			fprintf(stdout,
				"%-125s : %-12s : %s : %-3d : %s\n", name,
				get_mode(), result, errno,
				strerror(errno));
			fflush(stdout);
		}
	} else {
		result = "FAIL";
		fprintf(stdout, "%-125s : %-12s : %s : %-3d : %s\n", name,
			get_mode(), result, errno, strerror(errno));
		fflush(stdout);
	}
	fprintf(log_stream, "%-125s : %-12s : %s : %d : %-3d : %s\n", name,
		get_mode(), result, ret_value, errno, strerror(errno));
}

/* Verifies the output of a getsockopt() call that is expected to be successful,
 * and hence also checks the ouput parameters *optval and *optlen. */
static int print_getsockopt_macro(int test, char *exp, char *name,
				  int ret_value, int ret_errno,
				  long long int optval,
				  long long int optval_exp,
				  socklen_t optlen, socklen_t optlen_exp)
{
	char *result;
	if (ret_value == 0 && optval == optval_exp && optlen == optlen_exp) {
		result = "PASS";
		if (VERBOSE == 1) {
			printf("%-125s : %-12s : %s : ret = %d : optval = %d : optlen = %d\n",
			       name, get_mode(), result, ret_value,
			       optval, optlen);
			fflush(stdout);
		}
	} else {
		result = "FAIL";
		if (ret_value == 0) {
			printf("%-125s : %-12s : %s : ret = %d : optval = %d : optlen = %d\n",
			       name, get_mode(), result, ret_value,
			       optval, optlen);
		} else {
			printf("%-125s : %-12s : %s : ret = %d : %-3d : %s\n",
			       name, get_mode(), result, ret_value, errno,
			       strerror(errno));
		}
		fflush(stdout);
	}
	fprintf(log_stream, "%-125s : %-12s : %s : %d : %-3d : %s : optval = %d : optlen = %d\n",
		name, get_mode(), result, ret_value, errno, strerror(errno),
		optval, optlen);
}

#endif
