/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _RESULT_H
#define _RESULT_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netdb.h>
#include <netinet/in.h>
#include <exs.h>
#include <signal.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <ctype.h>

#define DUMMY_EXS 0

int VERBOSE = 0;

int mode = 0;

int timeout = 30;

#define TMP_PATH_MAX 100

char* exs_test_temp = "/tmp";
char* exs_test_fifo_1 = "exs_test_fifo1";
char* exs_test_fifo_2 = "exs_test_fifo2";
char exs_test_small_file[TMP_PATH_MAX];
char exs_test_large_file[TMP_PATH_MAX];

#include "inc/fault.h"
#include "inc/param.h"
#include "inc/print.h"
#include "cmdline.h"

#include "setup/exs_accept_setup.h"

#include "setup/exs_init_setup.h"

param *p;

pid_t child_pid;

static void run_server(param * p);

static pid_t run_test(int i, void *test_func)
{
	pid_t pid = fork();
	if (i == -1)
		return pid;	//invalid range provided
	if (pid == 0) {
		alarm(timeout);
		int test_id = i;
		p->test_number = test_id;
		pthread_t test_thread;
		if (pthread_create(&test_thread, NULL, test_func, p)) {
			fprintf(stderr, "Error creating thread\n");
		}
		if (pthread_join(test_thread, NULL)) {
			fprintf(stderr, "Error joining thread\n");
		}
	} else {
		int returnStatus;
		waitpid(pid, &returnStatus, 0);
	}
	return pid;
}

#include "setup/exs_accept_setup.h"
#include "setup/exs_bind_setup.h"
#include "setup/exs_blocking_accept_setup.h"
#include "setup/exs_blocking_close_setup.h"
#include "setup/exs_blocking_connect_setup.h"
#include "setup/exs_blocking_recv_setup.h"
#include "setup/exs_blocking_send_setup.h"
#include "setup/exs_close_setup.h"
#include "setup/exs_connect_setup.h"
#include "setup/exs_getsockopt_setup.h"
#include "setup/exs_init_setup.h"
#include "setup/exs_listen_setup.h"
#include "setup/exs_mderegister_setup.h"
#include "setup/exs_mregister_setup.h"
#include "setup/exs_qcreate_setup.h"
#include "setup/exs_qdelete_setup.h"
#include "setup/exs_qdequeue_setup.h"
#include "setup/exs_qmodify_setup.h"
#include "setup/exs_qstatus_setup.h"
#include "setup/exs_read_setup.h"
#include "setup/exs_recv_setup.h"
#include "setup/exs_send_setup.h"
#include "setup/exs_setsockopt_setup.h"
#include "setup/exs_shutdown_setup.h"
#include "setup/exs_socket_setup.h"
#include "setup/exs_write_setup.h"
#include "setup/exs_sendfile_setup.h"
#include "setup/exs_blocking_sendfile_setup.h"

static int list_tests(param * local_p, int i)
{
	if (strcmp(local_p->func, "./test_exs_accept") == 0)
		list_accept_tests(i);
	if (strcmp(local_p->func, "./test_exs_bind") == 0)
		list_bind_tests(i);
	if (strcmp(local_p->func, "./test_exs_blocking_accept") == 0)
		list_blocking_accept_tests(i);
	if (strcmp(local_p->func, "./test_exs_blocking_close") == 0)
		list_blocking_close_tests(i);
	if (strcmp(local_p->func, "./test_exs_blocking_connect") == 0)
		list_blocking_connect_tests(i);
	if (strcmp(local_p->func, "./test_exs_blocking_recv") == 0)
		list_blocking_recv_tests(i);
	if (strcmp(local_p->func, "./test_exs_blocking_send") == 0)
		list_blocking_send_tests(i);
	if (strcmp(local_p->func, "./test_exs_close") == 0)
		list_close_tests(i);
	if (strcmp(local_p->func, "./test_exs_connect") == 0)
		list_connect_tests(i);
	if (strcmp(local_p->func, "./test_exs_getsockopt") == 0)
		list_getsockopt_tests(i);
	if (strcmp(local_p->func, "./test_exs_init") == 0)
		list_init_tests(i);
	if (strcmp(local_p->func, "./test_exs_listen") == 0)
		list_listen_tests(i);
	if (strcmp(local_p->func, "./test_exs_mderegister") == 0)
		list_mderegister_tests(i);
	if (strcmp(local_p->func, "./test_exs_mregister") == 0)
		list_mregister_tests(i);
	if (strcmp(local_p->func, "./test_exs_qcreate") == 0)
		list_qcreate_tests(i);
	if (strcmp(local_p->func, "./test_exs_qdelete") == 0)
		list_qdelete_tests(i);
	if (strcmp(local_p->func, "./test_exs_qdequeue") == 0)
		list_qdequeue_tests(i);
	if (strcmp(local_p->func, "./test_exs_qmodify") == 0)
		list_qmodify_tests(i);
	if (strcmp(local_p->func, "./test_exs_qstatus") == 0)
		list_qstatus_tests(i);
	if (strcmp(local_p->func, "./test_exs_read") == 0)
		list_read_tests(i);
	if (strcmp(local_p->func, "./test_exs_recv") == 0)
		list_recv_tests(i);
	if (strcmp(local_p->func, "./test_exs_send") == 0)
		list_send_tests(i);
	if (strcmp(local_p->func, "./test_exs_setsockopt") == 0)
		list_setsockopt_tests(i);
	if (strcmp(local_p->func, "./test_exs_shutdown") == 0)
		list_shutdown_tests(i);
	if (strcmp(local_p->func, "./test_exs_socket") == 0)
		list_socket_tests(i);
	if (strcmp(local_p->func, "./test_exs_write") == 0)
		list_write_tests(i);
	if (strcmp(local_p->func, "./test_exs_sendfile") == 0)
		list_sendfile_tests(i);
	if (strcmp(local_p->func, "./test_exs_blocking_sendfile") == 0)
		list_blocking_sendfile_tests(i);
}

static int execute(int argc, char *argv[], void *test_func, int num_tests)
{
	p = calloc(1, sizeof(param));
	p->func = argv[0];	//"test_exs_init";

	/* get temporary directory */
	char* tmpenv = getenv("TMPDIR");
	if( tmpenv != NULL ) exs_test_temp = tmpenv;

	/* create temporary directory */
	char tmp[TMP_PATH_MAX];
	int n = snprintf(tmp, TMP_PATH_MAX, "%s/exs_test_XXXXXX", exs_test_temp);
	if ( n > TMP_PATH_MAX ){
		fprintf(stdout, "FAIL: $TMPDIR too large for buffer; exiting\n");
		return -1;
	}
	char* tmpdir = mkdtemp(tmp);

	/* create fifos */
	char fifo1[TMP_PATH_MAX];
	n = snprintf(fifo1, TMP_PATH_MAX, "%s/%s", tmpdir, exs_test_fifo_1);
	if ( n > TMP_PATH_MAX ){
		fprintf(stdout, "FAIL: $TMPDIR too large for buffer; exiting\n");
		return -1;
	}
	mkfifo(fifo1, 0600);

	char fifo2[TMP_PATH_MAX];
	n = snprintf(fifo2, TMP_PATH_MAX, "%s/%s", tmpdir, exs_test_fifo_2);
	if ( n > TMP_PATH_MAX ){
		fprintf(stdout, "FAIL: $TMPDIR too large for buffer; exiting\n");
		return -1;
	}
	mkfifo(fifo2, 0600);

	n = snprintf(exs_test_small_file, TMP_PATH_MAX, "%s/small.txt", tmpdir);
	if ( n > TMP_PATH_MAX ){
		fprintf(stdout, "FAIL: $TMPDIR too large for buffer; exiting\n");
		return -1;
	}

	n = snprintf(exs_test_large_file, TMP_PATH_MAX, "%s/large.txt", tmpdir);
	if ( n > TMP_PATH_MAX ){
		fprintf(stdout, "FAIL: $TMPDIR too large for buffer; exiting\n");
		return -1;
	}

    /** get command line options **/
	if (setup_options(argc, argv, p) != 0) {
		return 0;
	}

	/** list tests, then end **/
	if (p->list == 1) {
		int opt;
		if (p->list_opt != NULL && !isdigit(p->list_opt[0])) {
			fprintf(stdout, "-t option is not a number\n");
			exit(0);
		}

		if (p->list_opt == NULL)
			opt = 0;
		if (p->list_opt != NULL)
			opt = atoi(p->list_opt);

		list_tests(p, opt);
		exit(0);
	}

    /** run all tests (no tests specified) **/
	if (p->iter->value == 0) {
		int i;
		for (i = 1; i < num_tests + 1; i++) {
			p->test_number = i;

			if( strstr(argv[0], "sendfile") != NULL ){
				start_sendfile();
			}

			if (run_test(i, test_func) == 0)
				break;

			if( strstr(argv[0], "sendfile") != NULL ){
				end_sendfile();
			}
		}
	}
    /** run specified tests **/
	else {
		struct node *iter = p->iter;
		while (iter->next != NULL) {
			p->test_number = iter->value;

			if( strstr(argv[0], "sendfile") != NULL ){
				start_sendfile();
			}

			if (run_test(iter->value, test_func) == 0)
				break;

			if( strstr(argv[0], "sendfile") != NULL ){
				end_sendfile();
			}

			iter = iter->next;
		}
	}

	/* remove files */
	remove(fifo1);
	remove(fifo2);

	/* remove temp directory */
	rmdir(tmpdir);

	return 0;
}
#endif
