/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_ACCEPT_SETUP_H
#define _EXS_ACCEPT_SETUP_H

#include "../result.h"

#define ACCEPT_TESTS 8

enum get_bound_socket_flags {
	SOCKET_LISTEN = 0x1,
	SOCKET_NOBUFS = 0x2,
	SOCKET_ALL_FLAGS = 0x3,
};

static char *all_accept_tests[ACCEPT_TESTS] = {
	"1  : EPERM                   : exs_accept(fd, &acceptaddr, 1, flags, qh) [no exs_init]",
	"2  : EBADF                   : exs_accept(BAD_FD, &acceptaddr, 1, flags, qh)",
	"3  : EINVAL                  : exs_accept(fd, &acceptaddr, 0, flags, qh) [no exs_listen]",
	"4  : EINVAL                  : exs_accept(fd, &acceptaddr, NEGATIVE_ADDRVEC_CNT, 0, qh) [addrvect <= when non blocking]",
	"5  : EINVAL                  : exs_accept(fd, &acceptaddr, ZERO_ADDRVEC_CNT, EXS_BLOCK, qh) [addrvec_cnet != 1 when blocking]",
	"6  : EINVAL                  : exs_accept(fd, &acceptaddr, 1, BAD_FLAG, qh)",
	"7  : EINVAL                  : exs_accept(fd, &acceptaddr, 1, flags, INVALID_QHANDLE)",
	"8  : OK_EXS_ACCEPT           : exs_accept(fd, &acceptaddr, 1, flags, qh)"
};

static int list_accept_tests(int select)
{
	if (select < 0 || select > ACCEPT_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			ACCEPT_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < ACCEPT_TESTS; i++) {
			if (strcmp(all_accept_tests[i], "") != 0)
				fprintf(stdout, "%s\n",
					all_accept_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_accept_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_accept(param * p)
{
	char *name = "ok_exs_accept";

	struct addrinfo *info;
	struct addrinfo hints;
	memset(&hints, 0, sizeof(hints));
	hints.ai_family = p->family;
	hints.ai_socktype = p->socktype;
	hints.ai_protocol = p->protocol;
	hints.ai_flags = AI_PASSIVE;

	int r = getaddrinfo(p->port_name, p->port_num, &hints, &info);

	exs_acceptaddr_t acceptaddr;
	exs_event_t event[2];
	exs_qhandle_t qh;

	struct sockaddr_storage addr_store;
	socklen_t addrlen;
	addrlen = sizeof(addr_store);
	memset(&acceptaddr, 0, sizeof(acceptaddr));

	int ret = exs_accept(p->fd, &acceptaddr, 1, p->flags, p->qh);

	if (ret < 0) {
		print_error(name);
		return -1;
	}

	p->recv_fd = ret;

	return 0;
}

#endif
