/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_BLOCKING_SEND_SETUP_H
#define _EXS_BLOCKING_SEND_SETUP_H

#include "../result.h"

#define BLOCKING_SEND_TESTS 14

static char *all_blocking_send_tests[BLOCKING_SEND_TESTS] = {
	"1  : EPERM                   : exs_blocking_send(fd, send_buf, SEND_BUFSIZE, flags, mh) [no exs_init]",
	"2  : EBADF                   : exs_blocking_send(BAD_FD, send_buf, SEND_BUFSIZE, flags, mh)",
	"3  : ENOTCONN                : exs_blocking_send(NOT_CONNECTED_FD, send_buf, SEND_BUFSIZE, flags, mh)",
	"4  : EINVAL                  : exs_blocking_send(fd, NULL_BUFFER, SEND_BUFSIZE, flags, mh)",
	"5  : EINVAL                  : exs_blocking_send(fd, send_buf, SEND_BUFSIZE, EXS_UNSIGNALED | EXS_BLOCK , mh)",
	"6  : EOPNOTSUPP              : exs_blocking_send(fd, send_buf, SEND_BUFSIZE, BAD_FLAG, mh)",
	"",
	"8  : EPIPE                   : exs_blocking_send(p->fd, send_buf, SEND_BUFSIZE, flags, mh)",
	"",
	"",
	"",
	"",
	"",
	"14 : OK_EXS_BLOCKING_SEND    : exs_blocking_send(fd, send_buf, SEND_BUFSIZE, flags, mh)"
};

static int list_blocking_send_tests(int select)
{
	if (select < 0 || select > BLOCKING_SEND_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			BLOCKING_SEND_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < BLOCKING_SEND_TESTS; i++) {
			if (strcmp(all_blocking_send_tests[i], "") != 0)
				fprintf(stdout, "%s\n",
					all_blocking_send_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n",
			all_blocking_send_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_blocking_send(param * p)
{
	char *name = "ok_exs_blocking_send";

	int ret =
	    exs_blocking_send(p->fd, p->send_buf, p->SEND_BUFSIZE,
			      p->flags, p->mh);

	if (ret < p->SEND_BUFSIZE) {
		print_error(name);
		return -1;
	}
	return ret;

}

#endif
