/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_MREGISTER_SETUP_H
#define _EXS_MREGISTER_SETUP_H

#include "../result.h"

#define MREGISTER_TESTS 6

static char *all_mregister_tests[MREGISTER_TESTS] = {
	"1  : EPERM                   : exs_mregister(buf, BUFSIZE, 0) [exs_init]",
	"",
	"3  : EINVAL                  : exs_mregister(NULL, BUFSIZE, 0)",
	"4  : EINVAL                  : exs_mregister(buf, ZERO_SIZE, 0) [size <= 0]",
	"5  : EINVAL                  : exs_mregister(buf, BUFSIZE, BAD_FLAG)",
	"6  : OK_EXS_MREGISTER        : exs_mregister(buf, BUFSIZE, 0)"
};

static int list_mregister_tests(int select)
{
	if (select < 0 || select > MREGISTER_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			MREGISTER_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < MREGISTER_TESTS; i++) {
			if (strcmp(all_blocking_send_tests[i], "") != 0)
				fprintf(stdout, "%s\n",
					all_mregister_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_mregister_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_mregister(param * p)
{
	char *name = "ok_exs_mregister";

	exs_mhandle_t send_ret =
	    exs_mregister(p->send_buf, p->SEND_BUFSIZE, 0);
	exs_mhandle_t recv_ret =
	    exs_mregister(p->recv_buf, p->RECV_BUFSIZE, 0);

	if (send_ret == EXS_MHANDLE_INVALID
	    || recv_ret == EXS_MHANDLE_INVALID) {
		print_error(name);
		return -1;
	}

	p->send_mh = send_ret;
	p->recv_mh = recv_ret;

	return 0;
}

#endif
