/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifndef _EXS_SOCKET_SETUP_H
#define _EXS_SOCKET_SETUP_H

#include "../result.h"

#define SOCKET_TESTS 16

static char *all_socket_tests[SOCKET_TESTS] = {
	"1  : EPERM                   : exs_socket(PF_INET, SOCK_STREAM, IPPROTO_TCP) [no exs_init]",
	"2  : ESOCKTNOSUPPORT         : exs_socket(PF_INET,           -1,      IPPROTO_TCP)",
	"3  : ESOCKTNOSUPPORT         : exs_socket(PF_INET,           -1,     IPPROTO_SCTP)",
	"4  : EPROTONOSUPPORT         : exs_socket(PF_INET,  SOCK_STREAM,               -1)",
	"5  : PASS                    : exs_socket(PF_INET,  SOCK_STREAM,      IPPROTO_TCP)",
	"6  : EPROTONOSUPPORT         : exs_socket(PF_INET,  SOCK_STREAM,               -1)",
	"7  : EPROTONOSUPPORT         : exs_socket(PF_INET,  SOCK_STREAM,     IPPROTO_SCTP)",
	"8  : PASS                    : exs_socket(PF_INET6, SOCK_STREAM,      IPPROTO_TCP)",
	"9  : EPROTONOSUPPORT         : exs_socket(PF_INET6, SOCK_STREAM,               -1)",
	"10 : EPROTONOSUPPORT         : exs_socket(PF_INET6, SOCK_STREAM,     IPPROTO_SCTP)",
	"11 : PASS                    : exs_socket(PF_INET6, SOCK_SEQPACKET,  IPPROTO_SCTP)",
	"12 : EPROTONOSUPPORT         : exs_socket(PF_INET6, SOCK_SEQPACKET,            -1)",
	"13 : EPROTONOSUPPORT         : exs_socket(PF_INET6, SOCK_SEQPACKET,   IPPROTO_TCP)",
	"14 : PASS                    : exs_socket(PF_INET6, SOCK_SEQPACKET,  IPPROTO_SCTP)",
	"15 : EPROTONOSUPPORT         : exs_socket(PF_INET6, SOCK_SEQPACKET,            -1)",
	"16 : OK_EXS_SOCKET           : exs_socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)"
};

static int list_socket_tests(int select)
{
	if (select < 0 || select > SOCKET_TESTS) {
		fprintf(stdout, "Invalid -t test number (1 - %d)\n",
			SOCKET_TESTS);
		exit(0);
	}

	if (select == 0) {
		int i;
		for (i = 0; i < SOCKET_TESTS; i++) {
			if (strcmp(all_socket_tests[i], "") != 0)
				fprintf(stdout, "%s\n",
					all_socket_tests[i]);
		}
	} else {
		fprintf(stdout, "%s\n", all_socket_tests[select - 1]);
	}

	exit(0);
}

static int ok_exs_socket(param * p)
{
	char *name = "ok_exs_socket";

	p->family = PF_INET;
	p->socktype = SOCK_STREAM;
	p->protocol = IPPROTO_TCP;

	struct addrinfo *info;
	struct addrinfo hints;
	memset(&hints, 0, sizeof(hints));
	hints.ai_family = p->family;
	hints.ai_socktype = p->socktype;
	hints.ai_protocol = p->protocol;
	hints.ai_flags = AI_PASSIVE;

	int r = getaddrinfo(p->addr, p->port, &hints, &info);

	int ret = exs_socket(info->ai_family, info->ai_socktype, 0);

	if (ret < 0) {
		print_error(name);
		return -1;
	}

	p->fd = ret;

	return 0;
}

#endif
