/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_init_setup.h"
#include "setup/exs_mregister_setup.h"

#define BAD_FLAG 2		//TODO

void *test_mregister(void *test_void_ptr)
{
	setup_fault_catcher(p, getpid());

	param *p = (param *) test_void_ptr;
	int test = p->test_number;

	/* EPERM: exs_init has not been called */
	if (test == 1) {
		p->test_value = "EPERM";
		p->test_name = all_mregister_tests[test - 1];
		exs_mhandle_t ret =
		    exs_mregister(p->send_buf, p->SEND_BUFSIZE, 0);
		print_mhandle_macro(test, p->test_value, p->test_name, ret,
				    EPERM);
	} else if (test == 2) {	//EACCESS - memory permissions

	}

	/* EINVAL: addr is NULL */
	else if (test == 3) {
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		p->test_value = "EINVAL";
		p->test_name = all_mregister_tests[test - 1];
		exs_mhandle_t ret =
		    exs_mregister(NULL, p->SEND_BUFSIZE, 0);
		print_mhandle_macro(test, p->test_value, p->test_name, ret,
				    EINVAL);
	}

	/* EINVAL: size <= 0 */
	else if (test == 4) {
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		p->test_value = "EINVAL";
		p->test_name = all_mregister_tests[test - 1];
		exs_mhandle_t ret = exs_mregister(p->send_buf, 0, 0);
		print_mhandle_macro(test, p->test_value, p->test_name, ret,
				    EINVAL);
	}


	/* EINVAL: invalid flags */
	else if (test == 5) {
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);
		p->test_value = "EINVAL";
		p->test_name = all_mregister_tests[test - 1];
		exs_mhandle_t ret =
		    exs_mregister(p->send_buf, p->SEND_BUFSIZE, -1);
		print_mhandle_macro(test, p->test_value, p->test_name, ret,
				    EINVAL);
	}

	/* OK_EXS_MREGISTER */
	else if (test == 6) {
		p->test_value = "OK_EXS_MREGISTER";
		p->test_name = all_mregister_tests[test - 1];
		if (ok_exs_init(p) < 0)
			exit(EXIT_FAILURE);

		int ret = ok_exs_mregister(p);

		if (ret == 0) {
			print_macro(test, p->test_value, p->test_name, ret,
				    0, 0);
		} else {
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, ENOTSUP);
		}
	}

	/** test number not valid **/
	else if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}
}

int main(int argc, char *argv[])
{				//arv[1] first arg from test_all.sh
	execute(argc, argv, test_mregister, MREGISTER_TESTS);
}
