/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#include "result.h"
#include "setup/exs_init_setup.h"
#include "setup/exs_socket_setup.h"
#include "setup/exs_socket_params.h"

void *test_socket(void *test_void_ptr)
{
	setup_fault_catcher(p, getpid());

	param *p = (param *) test_void_ptr;
	int test = p->test_number;
	int ret = 0;

	/* EPERM: exs_init has not been called */
	if (test == 1) {
		p->test_value = "EPERM";
		p->test_name = all_socket_tests[test - 1];

		ret = exs_socket(p->family, p->socktype, p->protocol);	/*segfault */
		print_macro(test, p->test_value, p->test_name, ret, -1,
			    EPERM);
		if (ret == -1) {
			exit(EXIT_SUCCESS);
		} else {
			exit(EXIT_FAILURE);
		}
	}

	if (ok_exs_init(p) < 0)
		exit(EXIT_FAILURE);

	/* run through various test options */
	int i;
	for (i = 0; i < NUM_TESTS; i++) {
		int family = exs_socket_params[i][0];
		int socktype = exs_socket_params[i][1];
		int protocol = exs_socket_params[i][2];
		int error = exs_socket_params[i][3];
		/*p->test_name = exs_socket_params_string[i][0]; */
		p->test_name = all_socket_tests[test - 1];
		char *value = exs_socket_params_value[i][0];
		p->test_value = value;

		int ret = 0;
		if (test == i + 1) {

			ret = exs_socket(family, socktype, protocol);	/*segfault */

			if (ret >= 0)
				ret = 0;	/*return value is fd */
			else
				ret = -1;

			if (error == PASS) {
				print_macro(test, value, p->test_name, ret,
					    0, error);
				if (ret == 0) {
					exit(EXIT_SUCCESS);
				} else {
					exit(EXIT_FAILURE);
				}
			} else {
				print_macro(test, value, p->test_name, ret,
					    -1, error);
				if (ret == -1) {
					exit(EXIT_SUCCESS);
				} else {
					exit(EXIT_FAILURE);
				}
			}
		}
	}

	/* OK_EXS_SOCKET */
	if (test == NUM_TESTS + 1) {
		p->test_value = "OK_EXS_SOCKET";
		p->test_name = all_socket_tests[test - 1];

		int ret;
		if (ret = ok_exs_socket(p) < 0) {
			print_macro(test, p->test_value, p->test_name, ret,
				    -1, ENOTSUP);
			exit(EXIT_FAILURE);
		} else {
			print_macro(test, p->test_value, p->test_name, ret,
				    0, 0);
			exit(EXIT_SUCCESS);
		}
	}

	/** test number not valid **/
	if (test != 0) {
		fprintf(stdout, "Invalid Test Number: %d\n", test);
	}
}

int main(int argc, char *argv[])
{
	execute(argc, argv, test_socket, SOCKET_TESTS);
}
